@@ -9,7 +9,7 @@ if (getRversion() >= "3.5.0")
 function(...) {
 
   packageStartupMessage("\n",
-      "lessR 4.2.4                         feedback: gerbing@pdx.edu \n",
+      "lessR 4.2.5                         feedback: gerbing@pdx.edu \n",
       "--------------------------------------------------------------\n",
       "> d <- Read(\"\")   Read text, Excel, SPSS, SAS, or R data file\n",
       "  d is default data frame, data= in analysis routines optional\n",
@@ -20,7 +20,7 @@ function(...) {
       "  Enter:  browseVignettes(\"lessR\")\n\n",
       "View changes in this and recent versions of lessR.\n",
       "  Enter: news(package=\"lessR\")\n\n",
-      "**New Feature**: Interactive analysis of your data\n",
+      "**Newly Revised**: Interactive data analysis.\n",
       "  Enter: interact()\n")
 
   options(warn = -1)  # suppress warnings while bin.width, etc., allowed
@@ -199,21 +199,20 @@ function(...) {
 
 
 # maximum number of decimal digits in the values of a variable
-# only called by .getdigits(), which follows
+# takes much time, quite noticeable for large data sets, so restrict n
 .max.dd <- function(x) {
 
- n.dec <-function(xn) {
-    xc <- format(xn)  # as.character(51.45-48.98) does not work
-    nchar(xc)
-    ipos <- 0
-    for (i in 1:nchar(xc)) if (substr(xc,i,i)==".") ipos <- i
-    n.dec <- ifelse (ipos > 0, nchar(xc)-ipos, 0)
-    return(n.dec)
-  }
-
   max.dd <- 0
-  for (i in 1:length(x))
-    if (!is.na(x[i])) if (n.dec(x[i]) > max.dd) max.dd <- n.dec(x[i])
+  n.reps <- min(500, length(x))
+  for (i in 1:n.reps) {  # length(x) is number of data values
+    if (!is.na(x[i])) {
+      xc <- format(x[i])  # as.character(51.45-48.98) does not work
+      ipos <- 0
+      for (i in 1:nchar(xc)) if (substr(xc,i,i)==".") ipos <- i
+      n.dec <- ifelse (ipos > 0, nchar(xc)-ipos, 0)
+      if (n.dec > max.dd) max.dd <- n.dec
+    }
+  }
 
   return(max.dd)
 }
@@ -1785,101 +1784,6 @@ function(lab, labcex, cut, nm, var.nm, units) {
 }
 
 
-# parser called by .reg.Rmd() to process a file of marked text
-.toRmd <- function(f.name, path, n.pred=-1,
-                   explain=NULL, results=NULL, interpret=NULL,
-                   res_sort="") {
-# path <- getwd()
-  if (!is.null(path)) {  # a system file, spec includes full reference
-    f.name <- file.path(path, f.name) 
-  }
-  txt <- readLines(f.name, skipNul=TRUE)  # read all lines in the file
-
-  the.ln <- NULL
-  i.line <- 0
-  while (i.line < length(txt)) {
-    i.line <- i.line + 1
-    ln <- paste(txt[i.line], " ", sep="")  # the ith line of vector txt
-
-    if (substr(ln, 1, 1) == "%") { next }  # comment, skip and go to next line 
-
-    cl <- gregexec("`", ln)
-    if (cl[[1]][1] != -1) {  # at least 1 var found in the.ln
-      cols <- cl[[1]][1,]  # columns that contain vars marks
-      n.sym <- length(cols) / 2  # symbol mark comes in pairs
-      ind <- (1:length(cols)) %% 2
-      c.evn <- cols[ind==0]  # even indices cols
-      c.odd <- cols[ind==1]  # odd indices cols
-      c.beg <- c(1, c.evn+1)
-      c.end <- c(c.odd-1, nchar(ln))
-
-      # sym is symbol, whatever is between ` and `
-      for (i.sym in 1:n.sym) {  # process each var within the ith line
-        chr <- substr(ln, c.beg[i.sym], c.end[i.sym])
-        sym <- substr(ln, c.odd[i.sym]+1, c.evn[i.sym]-1)
-
-        if (substr(sym, 1, 3) == "end") { next }  # end if-block 
-
-        else  # a function
-          if (substr(sym, 1,3) == "eq." || substr(sym, 1,4) == "out." ||
-              substr(sym, 1,5) == "intr." || substr(sym, 1,5) == "plot." ||
-              substr(sym, 1,3) == "in.")
-            the.ln <- paste(the.ln, chr, eval.parent(as.name(sym), n=1)(),
-                            sep="")
-
-        else  # inline R expression
-          if (substr(sym, 1,2) == "r ") {
-            sym <- paste("`", sym, "`", sep="") 
-            the.ln <- paste(the.ln, chr, sym, sep="")
-          }
-
-        else  # assignment statement
-          if (grepl("<-", sym, fixed=TRUE))
-            the.ln <- paste(the.ln, "\n```{r echo=FALSE}\n", sym, "\n", "```",
-                            sep="")
-        
-        else  # header
-          if (substr(sym, 1,1) == "#")
-            the.ln <- paste(the.ln, chr, sym, "\n", sep="")
-
-        else  # if block
-          if (substr(sym, 1,2) == "if") {
-            delete <- FALSE
-            if (grepl("explain", sym)) if (!explain) delete <- TRUE
-            if (grepl("results", sym)) if (!results) delete <- TRUE
-            if (grepl("interpret", sym)) if (!interpret) delete <- TRUE
-            if (grepl("n.pred=1", sym)) if (n.pred>1) delete <- TRUE
-            if (grepl("n.pred>1", sym)) if (n.pred==1) delete <- TRUE
-            if (grepl("n.pred>2", sym)) if (n.pred<=2) delete <- TRUE
-            if (grepl("cooks", sym)) if (res_sort != "cooks") delete <- TRUE
-            if (delete) {
-              gone <- txt[i.line]
-              while (substr(gone, 1,4) != "`end") {
-                i.line <- i.line + 1
-                gone <- txt[i.line]
-              }
-            }
-          } 
-
-        else  # a variable with an assigned value, such as a variable name
-          the.ln <- paste(the.ln, chr, eval.parent(as.name(sym), n=1), sep="")
-
-        if (i.sym == n.sym) {  # get any text past the last sym
-          chr <- substr(ln, c.beg[n.sym+1], c.end[n.sym+1])
-          the.ln <- paste(the.ln, chr, sep="")
-         }
-
-      }  # end each var one at a time on indx
-    }  # end found vars on indx
-
-    else { # no vars in the line
-      if (ln == " ") ln <- "\n\n"
-      the.ln <- paste(the.ln, ln, sep="")
-    }
-  }  # end line-by-line processing
-
-  return(the.ln)
-}
 
 
 .prntbl <- function(x, digits_d=2, cut=0, cc="-", cors=FALSE,
