@@ -44,7 +44,7 @@ Plot(x, y)
 Plot(x, y, smooth=TRUE, fit="lm")
 
 ## ---------------------------------------------------------------------------------------------------------------------
-Plot(x, y, trans=0.95)
+Plot(x, y, transparency=0.95)
 
 ## ---------------------------------------------------------------------------------------------------------------------
 Plot(x, y, n_bins=5)
