@@ -7,73 +7,68 @@ library(lessR)
 style(lab_cex=1.2, axis_cex=1, suggest=FALSE)
 
 clr.one <- list(
-  "#324E5C", "dodgerblue3", "cornflowerblue", "darkblue", "pink2", "red3",
-  "darkred", "darkorange2", "lightcoral", "salmon", "ivory", "wheat3",
-  "burlywood4", "sienna", "goldenrod2", "yellow2", "darkseagreen2",
-  "springgreen3", "seagreen4", "violetred", "lavender", "thistle3",
-  "lightcoral", "magenta3", "mediumorchid",
-  "black", "gray45", "gray75", "gray95", "white")
+  "#324E5C", "dodgerblue3", "cornflowerblue", "steelblue", "darkblue",
+  "pink2", "red3", "firebrick2", "darkred",
+  "violetred", "mediumorchid", "purple3",
+  "darkorange2", "salmon", "orange3", "sienna", "rosybrown", 
+  "wheat3", "goldenrod2", "khaki", "yellow2",
+  "darkseagreen2", "springgreen3", "seagreen4", "darkgreen",
+  "black", "gray45", "slategray4", "gray75", "snow3", "gray95",
+  "lavender", "ivory2", "aliceblue", "white")
 
-clr.fit <- c("#5C4032", clr.one[2:length(clr.one)])
-
-clr.edge <- list("off", "#8496AF", "black", "gray50", "gray75", "white", 
-  "darkblue", "darkred", "darkgreen", "rosybrown2", "slategray2", "thistle1",
-  "coral", "gold", "ivory")
+clr.edge <- list("off", "black", "gray50", "gray75", "white", "ivory", 
+  "darkblue", "darkred", "darkgreen", "rosybrown2", "bisque", 
+  "slategray2", "aliceblue", "thistle1", "coral", "gold")
 
 clr.qual <- list("reds", "rusts", "browns", "olives", "greens",
   "emeralds", "turquoises", "aquas", "blues", "purples", "violets",
   "magentas", "grays")
 
+clr.fit <- c("#5C4032", clr.one[2:length(clr.one)])
+
+addResourcePath("shiny_dir", system.file("shiny_apps", package="lessR"))
+
 
 ui <- fluidPage(
-  tags$style(type='text/css', "
-             label {font-size: 1em; font-weight: bold;}           
-             hr {border-top: .10em solid #8f8f8f;}
-             .nav-tabs {font-size: 1.1em;}
-             .checkbox label {font-weight: bold;}
-             .soft {margin-bottom:.25em; font-size:1em; font-style:italic;}
-             .hp {color:#808080; font-size: .5em;}
-             .head {font-size:.85em; margin-top: -1em; color:#8f8f8f}
-             "),
-
-  titlePanel("Analysis"),
+tags$head(tags$link(rel="stylesheet", href="shiny_dir/styles.css")),
 
   tabsetPanel(
+
     tabPanel("Data",
-      headerPanel(div("Upload a text (.csv, .txt) or Excel file", class="hp")),
+      titlePanel(div("Upload a text (.csv, .txt) or Excel file", id="hp")),
 
       sidebarLayout(
         sidebarPanel(
-          radioButtons("fType", "Format", c("Excel"="Excel", "Text"="Text")),
+
+          radioButtons("fType", HTML("<h5 class='soft'>Format</h5>"), 
+                       c("Excel"="Excel", "Text"="Text")),
           conditionalPanel(condition="input.fType == 'Text'",
             radioButtons("sep", HTML("<h5 class='soft'>Separator</h5>"),
                          c(Comma=",", Semicolon=";", Tab="\t"), ","),
             radioButtons("decimal", HTML("<h5 class='soft'>Decimal</h5>"),
                          c("Point"=".", "Comma"=",")),
-            tags$br()
           ),
 
-          radioButtons("fSource", "Source", c("local"="local", "web"="web")),
+          radioButtons("fSource", HTML("<h5 class='soft'>Source</h5>"), 
+                       c("Local"="local", "Web"="web")),
           conditionalPanel(condition="input.fSource == 'local'",
             fileInput("myFile", "Locate your data file",
-                      accept=c(".csv", ".txt", ".xlsx", ".xlsm"))
+                      accept=c(".csv", ".txt", ".xlsx", ".xlsm")),
           ),
           conditionalPanel(condition="input.fSource == 'web'",
-            textInput("myURL", "web address"),
+            textInput("myURL", "Web address of data file"),
             actionButton("submitURL", "Submit")
           ),
 
           textOutput("nrows"),
           textOutput("ncols"),
-          tags$br(),
-          radioButtons("show", "Rows to display",
-                       c("First 10"="head", "Last 10"="tail", "All"="all"))
+          uiOutput("d.show"),
 
         ),  # end sidbarPanel
 
         mainPanel(
-          tableOutput("contents"),
-          tags$style(type="text/css", "#contents {font-size: .95em;}")
+          tableOutput("d.table"),
+          tags$style(type="text/css", "#d.table {font-size: .95em;}")
         )
 
       )  # end sidbarLayout
@@ -82,72 +77,78 @@ ui <- fluidPage(
 
     tabPanel("Scatterplot",
       pageWithSidebar(
-        headerPanel(div("2-Dimensional Scatterplot", class="hp")),
+        titlePanel(""),
 
         sidebarPanel(
           selectInput('x.col', 'x Variable', ""),
           selectInput('y.col', 'y Variable', "", selected=""),
 
-          checkboxInput("do_by", "add a by variable", FALSE),
+          checkboxInput("do_by", div("by variable", class="view"), FALSE),
           conditionalPanel(condition="input.do_by == true",
             selectInput("by.col", "by Variable", "", selected=""),
-            selectInput("myFill2", "fill",
+            selectInput("myFill_by", "fill",
               choices=list("Qualitative"=list("hues"), "Sequential"=clr.qual)),
           ),
 
-          checkboxInput("do_size", "add a size variable", FALSE),
+          checkboxInput("do_size", div("size variable", class="view"), FALSE),
           conditionalPanel(condition="input.do_size == true",
             selectInput('size.col', 'size Variable', "", selected=""),
           ),
 
           tags$hr(),
-          h4(div("Colors, Size, & Shape", class="head")),
-          checkboxInput("do_geom", "view options", FALSE),
+          checkboxInput("do_geom", div("Points", class="view"), FALSE),
           conditionalPanel(condition="input.do_geom == true",
             conditionalPanel(condition="input.do_by == false",
               selectInput("myFill", "fill", choices=clr.one),
               selectInput("myColor", label="color", choices=clr.edge),
             ),
-            sliderInput("myTrans", label="trans", min=0, max=1, value=0),
+            sliderInput("myTrans", "transparency", min=0, max=1, value=0),
             conditionalPanel(condition="input.do_size == false",
               sliderInput("mySize", "size", min=0.0, max=5, value=1.25,
                           step=0.25)),
-            selectInput("myShape", "shape",
+            conditionalPanel(condition="input.do_size == false",
+              selectInput("myShape", "shape",
               choices=list("circle", "square", "diamond", "triup", "tridown")),
+            ),
           ),
 
           tags$hr(),
-          h4(div("Fit Line, Ellipse, & Outliers", class="head")),
-          checkboxInput("do_FlEO", "view options", FALSE),
+          checkboxInput("do_FlEO", 
+                        div("Line, Ellipse, Outliers", class="view"), FALSE),
           conditionalPanel(condition="input.do_FlEO == true",
             conditionalPanel(condition="input.do_by == false",
               checkboxInput("myEnhance", "enhance", value=FALSE),
+              checkboxInput("myAddMeans", "add=\"means\"", value=FALSE),
             ),
-            checkboxInput("myErrors", "plot_errors", value=FALSE),
             selectInput("myFit", "fit",
               choices=list("off", "loess", "lm", "exp", "quad", "null")),
             conditionalPanel(condition="input.myFit != 'off'",
-              selectInput("myFitClr", "fit_color", choices=clr.fit),
+              checkboxInput("myErrors", "plot_errors", value=FALSE),
+              conditionalPanel(condition="input.do_by == false",
+                selectInput("myFitClr", "fit_color", choices=clr.fit),
+              ),
               sliderInput("myFitSE", "fit_se", min=0, max=0.99, value=0.95),
             ),
             sliderInput("myEllipse", "ellipse", min=0, max=0.99, value=0),
             conditionalPanel(condition="input.do_by == false",
-              sliderInput("myMDcut", "MD_cut", min=0, max=12, value=0),
+              sliderInput("myMDcut", "MD_cut", min=0, max=15, value=0),
+              conditionalPanel(condition="input.myMDcut > 0",
+                selectInput('ID.col', 'ID variable', "", selected=""),
+              ),
             ),
-            checkboxInput("myAddMeans", "add=\"means\"", value=FALSE),
           ),
 
           tags$hr(),
-          h4(div("Jitter Points", class="head")),
-          checkboxInput("do_jitter", "view options", FALSE),
+          checkboxInput("do_jitter", div("Jitter", class="view"), FALSE),
           conditionalPanel(condition="input.do_jitter == true",
-            sliderInput("myJitx", "jitter_x", min=0, max=2.5, value=0, step=0.10),
-            sliderInput("myJity", "jitter_y", min=0, max=2.5, value=0, step=0.10),
+            sliderInput("myJitx", "jitter_x", min=0, max=1000, value=0,
+                        step=0.5),  # need the max value set from data
+            sliderInput("myJity", "jitter_y", min=0, max=1000, value=0,
+                        step=0.5),
           ),
 
           tags$hr(),
-          h4(div("Rotate Axis Labels", class="head")),
-          checkboxInput("do_rotate", "view options", FALSE),
+          checkboxInput("do_rotate", div("Rotate Labels", class="view"), FALSE),
           conditionalPanel(condition="input.do_rotate == true",
             sliderInput("myRttx", "rotate_x", min=0, max=90, value=0, step=10),
             sliderInput("myRtty", "rotate_y", min=0, max=90, value=0, step=10),
@@ -155,8 +156,7 @@ ui <- fluidPage(
           ),
 
           tags$hr(),
-          h4(div("Save", class="head")),
-          checkboxInput("do_pdf", "view options", FALSE),
+          checkboxInput("do_pdf", div("Save", class="view"), FALSE),
           conditionalPanel(condition="input.do_pdf == true",
             sliderInput("w", "width (inches):", min=3, max=20, value=8),
             sliderInput("h", "height (inches):", min=3, max=20, value=6),
@@ -164,14 +164,19 @@ ui <- fluidPage(
             actionButton(inputId="btn_pdf", "Save"),
             tags$p(div("Save pdf file and R code file",
                   style="margin-top:.25em; margin-bottom:0em;")),
-          )
+          ),
+
+          tags$hr(),
+          checkboxInput("do_help", div("Help", class="view"), FALSE),
+
         ),  # end sidebarPanel
 
-        mainPanel(
-          plotOutput('myPlot'),
-          verbatimTextOutput("summary"),
-          plotOutput("saved_plot") 
-        )
+      mainPanel(
+        plotOutput('myPlot'),
+        verbatimTextOutput("summary"),
+        plotOutput("saved_plot"), 
+        textOutput("help")
+      )
 
       )  # end pageWithSidebar
   )  # end tabPanel 2
@@ -181,8 +186,8 @@ ui <- fluidPage(
 
 server <- function(input, output, session) {
 
-  vals <- reactiveValues()
-  vals$add <- NULL
+  v <- reactiveValues()
+  v$add <- NULL
 
 
 # ------- Read and Display Data -----------
@@ -197,14 +202,14 @@ server <- function(input, output, session) {
     if (input$fSource == "local") {
       shiny::req("input$myFile")
       myPath <- input$myFile$datapath
-      vals$theRead <- input$myFile$name
+      theRead <- input$myFile$name
     }
     if (input$fSource == "web") {
       url <- theURL()
       if (!(grepl("http://", url)))
         url <- paste("http://", url, sep="")
       myPath <- url
-       vals$theRead <- myPath
+       theRead <- myPath
     }
       
     shiny::req(myPath)
@@ -214,17 +219,18 @@ server <- function(input, output, session) {
         d <- read.xlsx(myPath, detectDates=TRUE)
       }
       else {
-        message("Excel file must have file type of .xlsx")
+        message("\n>>> Excel file must have file type of .xlsx <<<\n\n")
         stopApp()
       }
     }
       if (input$fType == "Text") { 
         if ((grepl(".csv", myPath, fixed=TRUE)) ||
             (grepl(".txt", myPath, fixed=TRUE))) {
-            d <- read.csv(myPath, sep=input$sep, dec=input$decimal)
+            d <- read.csv(myPath, sep=input$sep, dec=input$decimal,
+                          na.strings="")  # default is NOT a blank char missing
         }
         else {
-          message("Text file must have file type of .csv or .txt")
+          message("\n>>> Text file must have file type of .csv or .txt <<<\n\n")
           stopApp()
         }       
       }  # end fType is "Text"
@@ -234,21 +240,42 @@ server <- function(input, output, session) {
     shiny::updateSelectInput(session, inputId="y.col", label="y variable",
                       choices=c("Select a variable"="", names(d)))
     
-    output$nrows <- shiny::renderText({paste("Number of rows of data:", nrow(d))})
-    output$ncols <- shiny::renderText({paste("Number of variables:", ncol(d))})
-
     return(d)
   })  # end reactive()
 
 
-  output$contents <- renderTable({
-    if (input$show == "all")
+  output$d.show <- renderUI({
+    shiny::req(data())
+    output$nrows <- renderText({paste("Number of data rows:", nrow(data()))})
+    output$ncols <- renderText({paste("Number of variables:", ncol(data()))})
+    if (nrow(data()) > 10)
+      radioButtons("d.show", HTML("<h5 class='soft'>Rows to display</h5>"),
+                   c("First 10"="head", "Last 10"="tail", "Random 10"="random",
+                     "All"="all"))
+  })
+
+
+  output$d.table <- renderTable({
+    if (is.null(input$d.show)) 
       data()
-    else if (input$show == "head")
-      head(data(), n=10)
-    else if (input$show == "tail")
-      tail(data(), n=10)
-  })  # end renderTable
+    else {
+      nr <- min(11, nrow(data()))
+      if (nr == 11) {
+        if (input$d.show == "all")
+          data()
+        else if (input$d.show == "head")
+          head(data(), n=10)
+        else if (input$d.show == "tail")
+          tail(data(), n=10)
+        else if (input$d.show == "random") { 
+          dd <- data()
+          dd[.(random(10)), ]
+        }
+      }
+    }
+  }, striped=TRUE)  # end renderTable
+
+# -----------------------------------------
 
 
   observeEvent(input$do_by, {
@@ -263,7 +290,7 @@ server <- function(input, output, session) {
       message("A  by  variable partitions values of a categorical variable.\n",
               "Categorical variables have non-numeric values or, if numeric,\n",
               "  defined here as 10 or fewer unique values.\n\n",
-              "There are no categorical variables in this data set.")
+              "No categorical variables are present in this data set.")
       stopApp()
     }
 
@@ -281,24 +308,47 @@ server <- function(input, output, session) {
   })
 
 
+  observeEvent(input$myMDcut, {
+    shiny::req(input$myMDcut > 0)
+
+    d <- data()
+    is.unique <- sapply(d, function(x) {
+      l.x <- length(x)
+      lu.x <- length(unique(x))
+      x.unique <- ifelse (l.x == lu.x, TRUE, FALSE)      
+    })
+
+    if (all(!is.unique)) {
+      message("An  ID  variable must have unique values.\n",
+              "No variables have unique values in this data set.")
+      stopApp()
+    }
+
+    the.unique <- c("row.name", names(d)[is.unique])
+    shiny::updateSelectInput(session, inputId="ID.col", label="ID variable",
+                        choices=c("Var with unique values"="", the.unique),
+                        selected="row.name")
+  })
+
+
 # ------------ Enhance and Add ------------
 # -----------------------------------------
 
   observeEvent(input$myAddMeans, {
     if (input$myAddMeans)
-     vals$add <- "means"
+     v$add <- "means"
     else
-     vals$add <- NULL    
+     v$add <- NULL    
   })
 
 
   observeEvent(input$myEnhance, {
     if (input$myEnhance) {
-     vals$add <- "means"
+     v$add <- "means"
      add.val <- TRUE
     }
     else {
-     vals$add <- NULL
+     v$add <- NULL
      add.val <- FALSE
     }
     updateCheckboxInput(session, inputId="myAddMeans", "add=\"means\"",
@@ -315,7 +365,7 @@ server <- function(input, output, session) {
     if (!input$do_by) {
       MDc.val <- ifelse (input$myEnhance, 6, 0)
       updateSliderInput(session, inputId="myMDcut", "MD_cut",
-           min=0, max=12, value=MDc.val)
+           min=0, max=15, value=MDc.val)
     }
   })
  
@@ -333,7 +383,7 @@ server <- function(input, output, session) {
     shiny::req(y.name)
     y <- data()[, y.name]
 
-    in.fill <- ifelse (nchar(input$by.col)==0, input$myFill, input$myFill2)
+    in.fill <- ifelse (nchar(input$by.col)==0, input$myFill, input$myFill_by)
 
     by <- NULL
     lt <- NULL  # legend_title
@@ -341,7 +391,7 @@ server <- function(input, output, session) {
       by.name <- input$by.col
       shiny::req(by.name)
       by <- data()[, by.name]
-      in.fill <- input$myFill2  # color range selection
+      in.fill <- input$myFill_by  # color range selection
       lt <- by.name
     }
 
@@ -351,14 +401,37 @@ server <- function(input, output, session) {
       in.size <- data()[, input$size.col]
       size.name <- input$size.col
     }
-
     if (!input$do_size) in.size <- 1.4 * in.size  # scale up, as with axes
 
-     vals$p <- Plot(x, y, data=NULL, by=by,
-          fill=in.fill, color=input$myColor, trans=input$myTrans,
+    in.fit <- input$myFit
+    in.elp <- input$myEllipse
+    in.add <- v$add 
+    if (input$do_by) {  # choose MD then by var
+      in.MD <- 0
+      in.ID <- row.names(data())
+      if (input$myEnhance) {
+        in.fit <- "off" 
+        in.elp <- 0
+        in.add <- NULL
+      }
+    }
+    else {
+      in.MD <- input$myMDcut 
+      if (nchar(input$ID.col) != 0) {
+        if (input$ID.col != "row.name")
+          in.ID <- data()[, input$ID.col]
+        else
+          in.ID <- row.names(data())
+      }
+      else
+        in.ID <- row.names(data())
+    }
+
+     v$p <- Plot(x, y, data=NULL, by=by,
+          fill=in.fill, color=input$myColor, transparency=input$myTrans,
           size=in.size, shape=input$myShape, plot_errors=input$myErrors,
-          fit=input$myFit, fit_color=input$myFitClr, fit_se=input$myFitSE, 
-          ellipse=input$myEllipse, MD_cut=input$myMDcut, add=vals$add,
+          fit=in.fit, fit_color=input$myFitClr, fit_se=input$myFitSE, 
+          ellipse=in.elp, MD_cut=in.MD, ID=in.ID, add=in.add,
           enhance=input$myEnhance, jitter_x=input$myJitx, jitter_y=input$myJity,
           rotate_x=input$myRttx, rotate_y=input$myRtty, offset=input$myOff,  
           xlab=x.name, ylab=y.name, legend_title=lt, quiet=FALSE)
@@ -367,40 +440,40 @@ server <- function(input, output, session) {
                                                 in.fill == "hues") 
       p_color <- input$myColor == "off"
       p_trans <- input$myTrans == 0
-      p_size <- input$mySize == 1.25
+      p_by <- input$do_by == FALSE
+      p_size <- input$mySize==1.25 && length(in.size)==1
       p_shape <- input$myShape == "circle"
       p_errors <- input$myErrors == FALSE
-      p_fit <- input$myFit == "off"
+      p_fit <- in.fit == "off"
       p_fitclr <- input$myFitClr == getOption("fit_color")
       p_fitse <- input$myFitSE == 0.95
-      p_ellipse <- input$myEllipse == 0
-      p_MDcut <- input$myMDcut == 0
+      p_ellipse <- in.elp == 0
+      p_MDcut <- in.MD == 0
+      p_ID <- nchar(input$ID.col)==0 || input$ID.col=="row.name" || in.MD==0
       p_jitx <- input$myJitx == 0
       p_jity <- input$myJity == 0
-      p_add <- is.null(vals$add)
+      p_add <- is.null(in.add)
 
       # all enhance default settings must be TRUE for enhance to be TRUE
       p_enhance <- FALSE
-      if (input$myEnhance) {
+      if (input$myEnhance && in.MD > 0 && !is.null(v$add)) {
         if (input$myFit=="lm" && input$myEllipse==0.95
-            && input$myMDcut==6 && vals$add=="means")
+            && input$myMDcut==6 && v$add=="means")
           p_enhance <- TRUE
         else
           p_enhance <- FALSE
       }
 
-      # can have by or size but not both
-      if (input$do_by)
-        out <- paste("Plot(", x.name, ", ", y.name, ", by=", by.name, sep="")
-      else if (input$do_size)
-        out <- paste("Plot(", x.name, ", ", y.name, ", size=", size.name, sep="")
-      else
-        out <- paste("Plot(", x.name, ", ", y.name, sep="")
+      out <- paste("Plot(", x.name, ", ", y.name, sep="")
 
       if (!p_fill) out <- paste(out, ", fill=\"", in.fill, "\"", sep="")
       if (!p_color) out <- paste(out, ", color=\"", input$myColor, "\"", sep="")
-      if (!p_trans) out <- paste(out, ", trans=", input$myTrans, sep="")
-      if (!p_size) out <- paste(out, ", size=", input$mySize, sep="")
+      if (!p_trans) out <- paste(out, ", transparency=", input$myTrans, sep="")
+      if (!p_by) out <- paste(out, ", by=", by.name, sep="")
+      if (!p_size) {
+        sz.out <- ifelse (length(in.size)==1, input$mySize, input$size.col)
+        out <- paste(out, ", size=", sz.out, sep="")
+      }
       if (!p_shape) out <- paste(out, ", shape=\"", input$myShape, "\"", sep="")
       if (!p_errors) out <- paste(out, ", errors=", input$myErrors, sep="")
       if (!p_fitclr) out <- paste(out, ", fit_color=\"", input$myFitClr,
@@ -410,31 +483,33 @@ server <- function(input, output, session) {
       if (!p_jity) out <- paste(out, ", jitter_y=", input$myJity, sep="")
 
       if (!p_enhance) {
-        if (!p_fit) out <- paste(out, ", fit=\"", input$myFit, "\"", sep="")
-        if (!p_ellipse) out <- paste(out, ", ellipse=", input$myEllipse, sep="")
-        if (!p_MDcut) out <- paste(out, ", MDcut=", input$myMDcut, sep="")
-        if (!p_add) out <- paste(out, ", add=\"", vals$add, "\"", sep="")
+        if (!p_fit) out <- paste(out, ", fit=\"", in.fit, "\"", sep="")
+        if (!p_ellipse) out <- paste(out, ", ellipse=", in.elp, sep="")
+        if (!p_MDcut) out <- paste(out, ", MD_cut=", in.MD, sep="")
+        if (!p_add) out <- paste(out, ", add=\"", in.add, "\"", sep="")
       }
       else
         out <- paste(out, ", enhance=", p_enhance, sep="")
 
+      if (!p_ID) out <- paste(out, ", ID=\"", input$ID.col, "\"", sep="")
+
       out <- paste(out, ")", sep="")
       cat(out, "\n")
-      vals$code <- out  # save the code for a pdf file
+      v$code <- out  # save the code for a pdf file
   })  # end renderPlot
 
 
   # print stats
   output$summary <- renderPrint({
-    shiny::req(vals$p)
-    print(vals$p)
+    shiny::req(v$p)
+    print(v$p)
   })
 
 
   # clicking on the Save button generates a pdf file 
   plotInput <- eventReactive(input$btn_pdf, {
 
-    code <- vals$code
+    code <- v$code
 
     x.name <- input$x.col
     shiny::req(x.name)
@@ -452,7 +527,7 @@ server <- function(input, output, session) {
       by.name <- input$by.col
       shiny::req(by.name)
       by <- data()[, by.name]
-      in.fill <- input$myFill2  # color range selection
+      in.fill <- input$myFill_by  # color range selection
       lt <- by.name
     }
 
@@ -465,11 +540,35 @@ server <- function(input, output, session) {
     pdf.fname <- paste("plot_", x.name, y.name, ".pdf", sep="")
     pdf.path <- file.path(path.expand("~"), pdf.fname)
 
+    in.fit <- input$myFit
+    in.elp <- input$myEllipse
+    in.add <- v$add 
+    if (input$do_by) {  # choose MD then by var
+      in.MD <- 0
+      in.ID <- row.names(data())
+      if (input$myEnhance) {
+        in.fit <- "off" 
+        in.elp <- 0
+        in.add <- NULL
+      }
+    }
+    else {
+      in.MD <- input$myMDcut 
+      if (nchar(input$ID.col) != 0) {
+        if (input$ID.col != "row.name")
+          in.ID <- data()[, input$ID.col]
+        else
+          in.ID <- row.names(data())
+      }
+      else
+        in.ID <- row.names(data())
+    }
+
     Plot(x, y, data=NULL, by=by,
-         fill=in.fill, color=input$myColor, trans=input$myTrans,
+         fill=in.fill, color=input$myColor, transparency=input$myTrans,
          size=in.size, shape=input$myShape, plot_errors=input$myErrors,
-         fit=input$myFit, fit_color=input$myFitClr, fit_se=input$myFitSE, 
-         ellipse=input$myEllipse, MD_cut=input$myMDcut, add=vals$add,
+         fit=in.fit, fit_color=input$myFitClr, fit_se=input$myFitSE, 
+         ellipse=in.elp, MD_cut=in.MD, ID=in.ID, add=in.add,
          enhance=input$myEnhance, jitter_x=input$myJitx, jitter_y=input$myJity,
          rotate_x=input$myRttx, rotate_y=input$myRtty, offset=input$myOff,  
          xlab=x.name, ylab=y.name, legend_title=lt, quiet=TRUE,
@@ -488,48 +587,65 @@ server <- function(input, output, session) {
     message("---------------------------------------------")
     cat("\n")
 
-    if (input$do_cmt) {
-      cat("# The first line of any R/lessR session is always\n",
-          "library(\"lessR\")\n# Next read your data into R, then analysis\n\n",
-          file=r.path)
+    if (input$fSource == "web") {
+      url <- theURL()
+      if (!(grepl("http://", url)))
+        url <- paste("http://", url, sep="")
     }
 
-    read.path <- vals$theRead
-    read.code <- paste("d <- Read(\"", read.path, "\"", ")", sep="")
+    read.path <- ifelse (input$fSource == "local", input$myFile$name, url)
+    read.code <- paste("d <- Read(\"", read.path, "\")", sep="")
     is.local <- !grepl("http://", read.path, fixed=TRUE)
+
+    if (input$do_cmt)
+      cat("# The pound sign, #, indicates a comment, not part of R coding\n\n",
+          "# Begin a R/lessR session by loading the lessR functions ",
+          "from the library\n", sep="", file=r.path)
+      cat("library(\"lessR\")\n\n", file=r.path, append=TRUE)
+
+    if (input$do_cmt) {
+      cat("# Now read your data into an R data table, the data frame, here d",
+          "\n", sep="", file=r.path, append=TRUE)
+      if (is.local)
+        cat("# To browse for the data file, include nothing between the quotes",
+            "\n", sep="", file=r.path, append=TRUE)
+    }
+    if (is.local && input$do_cmt)
+      cat("d <- Read(\"\")\n\n", file=r.path, append=TRUE)
+
     if (is.local && input$do_cmt) {
+      cat("# For security, the path to your data file is not made available\n",
+          "# Another option replaces PATHtoFILE in the following with the path\n",
+          "# Remove the # sign in the first column and delete the previous ",
+          "Read()\n", sep="", file=r.path, append=TRUE)
       read.path <- file.path("PATHtoFILE", read.path) 
-      read.code <- paste("d <- Read(\"", read.path, "\"", ")", sep="")
-      cat("# For security, the path to your data file is",
-          "not available here\n", file=r.path, append=TRUE)
-      cat("# Either browse for the data file, with nothing",
-          "between the quotes,\n", file=r.path, append=TRUE)
-      cat("# or replace PATHtoFILE with the actual path\n\n",
-          file=r.path, append=TRUE)
-    }
-    cat("d <- Read(\"\"", ")\n",
-         file=r.path, append=(is.local && input$do_cmt), sep="")
-    if (is.local  &&  input$do_cmt) {
-      cat("#       or\n", file=r.path, append=TRUE)
-      cat(read.code, "\n\n", file=r.path, append=TRUE)
-    }
-    if (input$do_cmt) {
-      cat("# d is the default data frame name, otherwise\n", 
-          file=r.path, append=TRUE)
-      cat("# if reading into a data frame, with the specified name, add: ",
-          "data=\n\n", file=r.path, append=TRUE)
-    }
-    cat(code, "\n", file=r.path, append=TRUE)
-    if (input$do_cmt) {
-      cat("#       or\n", file=r.path, append=TRUE)
-      new.code <- sub(")", "", code, fixed=TRUE)
-      new.code <- paste(new.code, ", data=NAME)\n", sep="")
-      cat(new.code, file=r.path, append=TRUE)
+      read.code <- paste("# d <- Read(\"", read.path, "\")", sep="")
     }
+    cat(read.code, "\n\n", file=r.path, append=TRUE)
+
+    if (input$do_cmt)
+      cat("# Create the scatterplot and accompanying statistical analysis\n",
+          "# d is the default data frame name, so no need to specify\n",
+          sep="", file=r.path, append=TRUE)
+    cat(code, "\n\n", file=r.path, append=TRUE)
+
+    anlys <- "ScatterPlot()"
+    if (input$do_cmt)
+      cat("# If accessing data with a name other than d, must add  data=NAME\n",
+          paste("#   to the", anlys, "call, where NAME is the name of your",
+          "data frame"), "\n", sep="", file=r.path, append=TRUE)
 
   })
   output$saved_plot <- renderPlot({ plotInput() })
 
+
+  # access web page help file
+  output$help <- eventReactive(input$do_help, {
+    shiny::req(input$do_help)
+    fp <- system.file("shiny_apps/help/ScatterPlot.html", package="lessR")
+    browseURL(fp)
+  })
+
 }  # end server
 
 shinyApp(ui, server)
